@@ -14,7 +14,7 @@ my $prefix = __PACKAGE__;
 # this module import config
 #
 use Carp ();
-use Net::SNMP::Mixin::Util qw/idx2val push_error get_init_slot/;
+use Net::SNMP::Mixin::Util qw/idx2val hex2octet normalize_mac push_error get_init_slot/;
 
 #
 # this module export config
@@ -22,24 +22,28 @@ use Net::SNMP::Mixin::Util qw/idx2val push_error get_init_slot/;
 my @mixin_methods;
 
 BEGIN {
-  @mixin_methods = ( qw/ get_if_entries /);
+  @mixin_methods = (qw/ get_if_entries /);
 }
 
 use Sub::Exporter -setup => {
-  exports   => [@mixin_methods],
-  groups    => { default => [@mixin_methods], },
+  exports => [@mixin_methods],
+  groups  => { default => [@mixin_methods], },
 };
 
 #
 # SNMP oid constants used in this module
 #
 use constant {
-  IF_DESCR                    => '1.3.6.1.2.1.2.2.1.2',
-  IF_TYPE                     => '1.3.6.1.2.1.2.2.1.3',
-  IF_ADMIN_STATUS             => '1.3.6.1.2.1.2.2.1.7',
-  IF_OPER_STATUS              => '1.3.6.1.2.1.2.2.1.8',
-  IF_X_NAME                   => '1.3.6.1.2.1.31.1.1.1.1',
-  IF_X_ALIAS                  => '1.3.6.1.2.1.31.1.1.1.18',
+  IF_DESCR        => '1.3.6.1.2.1.2.2.1.2',
+  IF_TYPE         => '1.3.6.1.2.1.2.2.1.3',
+  IF_MTU          => '1.3.6.1.2.1.2.2.1.4',
+  IF_SPEED        => '1.3.6.1.2.1.2.2.1.5',
+  IF_PHYS_ADDRESS => '1.3.6.1.2.1.2.2.1.6',
+  IF_ADMIN_STATUS => '1.3.6.1.2.1.2.2.1.7',
+  IF_OPER_STATUS  => '1.3.6.1.2.1.2.2.1.8',
+  IF_X_NAME       => '1.3.6.1.2.1.31.1.1.1.1',
+  IF_X_HIGHSPEED  => '1.3.6.1.2.1.31.1.1.1.15',
+  IF_X_ALIAS      => '1.3.6.1.2.1.31.1.1.1.18',
 };
 
 =head1 NAME
@@ -48,11 +52,11 @@ Net::SNMP::Mixin::IfInfo - mixin class for interface related infos
 
 =head1 VERSION
 
-Version 0.07
+Version 1.00
 
 =cut
 
-our $VERSION = '0.07';
+our $VERSION = '1.00';
 
 =head1 SYNOPSIS
 
@@ -69,16 +73,15 @@ our $VERSION = '0.07';
 
   my $if_entries = $session->get_if_entries;
   foreach my $if_index ( sort { $a <=> $b } keys %$if_entries ) {
-    my $ifAdminStatus = $if_entries->{$if_index}->{ifAdminStatus} || 0;
-    my $ifOperStatus  = $if_entries->{$if_index}->{ifOperStatus}  || 0;
-    my $ifType        = $if_entries->{$if_index}->{ifType}        || 0;
-    my $ifName        = $if_entries->{$if_index}->{ifName}        || '';
-    my $ifDescr       = $if_entries->{$if_index}->{ifDescr}       || '';
-    my $ifAlias       = $if_entries->{$if_index}->{ifAlias}       || '';
+    my $ifAdminStatus = $if_entries->{$if_index}->{ifAdminStatus} // 0;
+    my $ifOperStatus  = $if_entries->{$if_index}->{ifOperStatus}  // 0;
+    my $ifType        = $if_entries->{$if_index}->{ifType}        // 0;
+    my $ifName        = $if_entries->{$if_index}->{ifName}        // '';
+    my $ifDescr       = $if_entries->{$if_index}->{ifDescr}       // '';
+    my $ifAlias       = $if_entries->{$if_index}->{ifAlias}       // '';
 
     printf "%5d  %1d/%1d  %-10.10s %-25.25s %-26.26s\n",
-      $if_index, $ifAdminStatus, $ifOperStatus,
-      $ifName,   $ifDescr,       $ifAlias;
+      $if_index, $ifAdminStatus, $ifOperStatus, $ifName, $ifDescr, $ifAlias;
   }
 
 =head1 DESCRIPTION
@@ -96,16 +99,20 @@ Returns parts ot the ifTable and ifXTable as a hash reference. The key is the co
   {
     INTEGER => {    # ifIndex as key
 
-      ifName        => DisplayString,    # an ifXTable entry
-      ifAlias       => DisplayString,    # an ifXTable entry
+      ifName      => DisplayString,    # ifXTable entries
+      ifAlias     => DisplayString,
+      ifHighspeed => GAUGE,
 
-      ifDescr       => DisplayString,    # an ifTable entry
-      ifType        => IANAifType,       # an ifTable entry
-      ifAdminStatus => INTEGER,          # an ifTable entry
-      ifOperStatus  => INTEGER,          # an ifTable entry
-    }
+      ifDescr       => DisplayString,    # ifTable entries
+      ifType        => IANAifType,
+      ifMtu         => INTEGER,
+      ifSpeed       => GAUGE,
+      ifPhysAddress => PhysAddress,
+      ifAdminStatus => INTEGER,
+      ifOperStatus  => INTEGER,
+      }
 
-    ... ,
+      ...,
   }
 
 =cut
@@ -117,31 +124,29 @@ sub get_if_entries {
   Carp::croak "$agent: '$prefix' not initialized,"
     unless $session->init_ok($prefix);
 
-  # stash for return values
+  # hash for return values
   my $result = {};
 
-  # the MIB tables are stored in {column}{row}{value} order
-  # but we return {row}{column}{value}
+  # the MIB tables are stored in {col}{idx}=value order but we return {idx}{col}=value
   #
-  # grab all rows from one random choosen column
-  my @rows = keys %{ $session->{$prefix}{ifInfo}{ifDescr} };
+  # grab all if indexes from one random choosen col
+  my @indexes = keys %{ $session->{$prefix}{ifInfo}{ifDescr} };
 
-  foreach my $row (@rows) {
+  # side effect: make a shallow copy for shallow values
+  foreach my $idx (@indexes) {
+    foreach my $col ( keys %{ $session->{$prefix}{ifInfo} } ) {
+      my $value = $session->{$prefix}{ifInfo}{$col}{$idx};
 
-    # loop over all columns
-    foreach my $column ( keys %{ $session->{$prefix}{ifInfo} } ) {
-
-      # rebuild in reverse order: result(row,column) = stash(column,row)
-      # side effect: make a shallow copy for shallow values
-
-      $result->{$row}{$column} =
-        $session->{$prefix}{ifInfo}{$column}{$row};
+      # convert back to OCTET-STRING, maybe already translated by Net::SNMP
+      if ( $col eq 'ifPhysAddress' ) {
+        $value = hex2octet($value);
+        $value = normalize_mac($value);
+      }
+      $result->{$idx}{$col} = $value;
     }
-
   }
 
   return $result;
-
 }
 
 =head1 INITIALIZATION
@@ -156,7 +161,7 @@ Fetch basic interface related snmp values from the host. Don't call this method
 # due to the asynchron nature, we don't know what init job is really the last, we decrement
 # the value after each callback
 #
-use constant THIS_INIT_JOBS => 2;
+use constant THIS_INIT_JOBS => 1;
 
 sub _init {
   my ( $session, $reload ) = @_;
@@ -164,8 +169,8 @@ sub _init {
 
   die "$agent: $prefix already initialized and reload not forced.\n"
     if exists get_init_slot($session)->{$prefix}
-      && get_init_slot($session)->{$prefix} == 0
-      && not $reload;
+    && get_init_slot($session)->{$prefix} == 0
+    && not $reload;
 
   # set number of async init jobs for proper initialization
   get_init_slot($session)->{$prefix} = THIS_INIT_JOBS;
@@ -173,11 +178,7 @@ sub _init {
   # populate the object with needed mib values
   #
   # map between ifIndexes -> ifDescr, ...
-  _fetch_if_table_entries($session);
-  return if $session->error;
-
-  # map between ifIndexes -> ifAlias, ...
-  _fetch_if_x_table_entries($session);
+  _fetch_if_entries($session);
   return if $session->error;
 
   return 1;
@@ -187,32 +188,33 @@ sub _init {
 
 Only for developers or maintainers.
 
-=head2 B<< _fetch_if_table_entries($session) >>
+=head2 B<< _fetch_if_entries($session) >>
 
-Get some MIB values from the ifTable needed by all other modules.
+Get selected MIB values from the ifTable and ifXTable.
 
 =cut
 
-sub _fetch_if_table_entries {
+sub _fetch_if_entries {
   my $session = shift;
   my $result;
 
-  # fetch all some entries from ifTable
+  # fetch selected entries from ifTable and ifXTable
   $result = $session->get_entries(
     -columns => [
-      IF_DESCR,  IF_TYPE, IF_ADMIN_STATUS, IF_OPER_STATUS,
-#      IF_X_NAME, IF_X_ALIAS,
+      IF_DESCR, IF_TYPE, IF_MTU, IF_SPEED, IF_PHYS_ADDRESS, IF_ADMIN_STATUS, IF_OPER_STATUS,
+      IF_X_NAME, IF_X_HIGHSPEED, IF_X_ALIAS,
     ],
 
     # define callback if in nonblocking mode
-    $session->nonblocking ? ( -callback => \&_if_table_entries_cb ) : (),
+    $session->nonblocking ? ( -callback => \&_if_entries_cb ) : (),
   );
 
-  unless (defined $result) {
+  unless ( defined $result ) {
+
     # Net::SNMP looses sometimes error messages in nonblocking
     # mode, so we save them in an extra buffer
     my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    push_error( $session, "$prefix: $err_msg" ) if $err_msg;
     return;
   }
 
@@ -221,103 +223,40 @@ sub _fetch_if_table_entries {
 
   # ok we are in synchronous mode, call the result mangling function
   # by hand
-  _if_table_entries_cb($session);
+  _if_entries_cb($session);
 
 }
 
-=head2 B<< _if_table_entries_cb($session) >>
+=head2 B<< _if_entries_cb($session) >>
 
-The callback for _fetch_if_table_entries
+The callback for _fetch_if_entries
 
 =cut
 
-sub _if_table_entries_cb {
+sub _if_entries_cb {
   my $session = shift;
   my $vbl     = $session->var_bind_list;
 
-  unless (defined $vbl) {
+  unless ( defined $vbl ) {
+
     # Net::SNMP looses sometimes error messages in nonblocking
     # mode, so we save them in an extra buffer
     my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    push_error( $session, "$prefix: $err_msg" ) if $err_msg;
     return;
   }
 
   # mangle result table to get plain idx->value
-  $session->{$prefix}{ifInfo}{ifDescr} = idx2val( $vbl, IF_DESCR );
-
-  $session->{$prefix}{ifInfo}{ifType}  = idx2val( $vbl, IF_TYPE );
-
-  $session->{$prefix}{ifInfo}{ifAdminStatus} =
-    idx2val( $vbl, IF_ADMIN_STATUS );
-
-  $session->{$prefix}{ifInfo}{ifOperStatus} =
-    idx2val( $vbl, IF_OPER_STATUS );
-
-  # this init job is finished
-  get_init_slot($session)->{$prefix}--;
-
-  return 1;
-}
-
-=head2 B<< _fetch_if_x_table_entries($session) >>
-
-Get some MIB values from the ifXTable needed by all other modules.
-
-=cut
-
-sub _fetch_if_x_table_entries {
-  my $session = shift;
-  my $result;
-
-  # fetch all some entries from ifXTable
-  $result = $session->get_entries(
-    -columns => [ IF_X_NAME, IF_X_ALIAS, ],
-
-    # define callback if in nonblocking mode
-    $session->nonblocking ? ( -callback => \&_if_x_table_entries_cb ) : (),
-  );
-
-  unless (defined $result) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
-    return;
-  }
-
-  # in nonblocking mode the callback will be called asynchronously
-  return 1 if $session->nonblocking;
-
-  # ok we are in synchronous mode, call the result mangling function
-  # by hand
-  _if_x_table_entries_cb($session);
-
-}
-
-=head2 B<< _if_x_table_entries_cb($session) >>
-
-The callback for _fetch_if_x_table_entries
-
-=cut
-
-sub _if_x_table_entries_cb {
-  my $session = shift;
-  my $vbl     = $session->var_bind_list;
-
-  unless (defined $vbl) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
-    return;
-  }
-
-  # mangle result table to get plain idx->value
-
-  $session->{$prefix}{ifInfo}{ifName}  = idx2val( $vbl, IF_X_NAME );
-
-  $session->{$prefix}{ifInfo}{ifAlias} = idx2val( $vbl, IF_X_ALIAS );
+  $session->{$prefix}{ifInfo}{ifDescr}       = idx2val( $vbl, IF_DESCR );
+  $session->{$prefix}{ifInfo}{ifType}        = idx2val( $vbl, IF_TYPE );
+  $session->{$prefix}{ifInfo}{ifMtu}         = idx2val( $vbl, IF_MTU );
+  $session->{$prefix}{ifInfo}{ifSpeed}       = idx2val( $vbl, IF_SPEED );
+  $session->{$prefix}{ifInfo}{ifPhysAddress} = idx2val( $vbl, IF_PHYS_ADDRESS );
+  $session->{$prefix}{ifInfo}{ifAdminStatus} = idx2val( $vbl, IF_ADMIN_STATUS );
+  $session->{$prefix}{ifInfo}{ifOperStatus}  = idx2val( $vbl, IF_OPER_STATUS );
+  $session->{$prefix}{ifInfo}{ifName}        = idx2val( $vbl, IF_X_NAME );
+  $session->{$prefix}{ifInfo}{ifHighspeed}   = idx2val( $vbl, IF_X_HIGHSPEED );
+  $session->{$prefix}{ifInfo}{ifAlias}       = idx2val( $vbl, IF_X_ALIAS );
 
   # this init job is finished
   get_init_slot($session)->{$prefix}--;
@@ -347,7 +286,7 @@ Karl Gaissmaier <karl.gaissmaier at uni-ulm.de>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008-2016 Karl Gaissmaier, all rights reserved.
+Copyright 2008-2021 Karl Gaissmaier, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
